--------------------------------------------------------------------------
-- Credit to Morgalm for initially adapting the Whisp FuBar plugin to LibDataBroker 
--------------------------------------------------------------------------

local mod = Whisp:NewModule("Broker Plugin", "AceEvent-3.0")
local tablet = AceLibrary("Tablet-2.0")
local ldb = LibStub:GetLibrary("LibDataBroker-1.1")
local L = LibStub("AceLocale-3.0"):GetLocale("Whisp")
mod.modName = L["Broker Plugin"]
mod.tablet = tablet
local brokerFrame = {}

local defaults = {
  char = {
    tooltipTimeFrame = "Session",      -- Which messages should be shown on the fubar tooltop
    tooltipSort = "Time",              -- Sort method for messages on the fubar tooltip
    frameWidth = 500,                  -- Width of the frame
  }
}

local options = {
  b2f = {
    type = "execute",
    name = L["Broker2FuBar options"],
    order = 200,
    desc = L["Open the Broker2FuBar options panel"],
    hidden = function() return not IsAddOnLoaded('Broker2FuBar') end,
    func = function() LibStub("AceAddon-3.0"):GetAddon("Broker2FuBar", true):OpenGUI() end,
  },
}

function mod:OnInitialize()
  self.db = Whisp.db:RegisterNamespace("Broker Plugin", defaults)
  brokerFrame = self:CreateMyFrame("BROKER", self.db.char.frameWidth)
  mod.brokerFrame = brokerFrame
  mod:CreateLDBObject()
  self.skinned = nil
end

function mod:OnEnable()
  mod:RegisterMessage("WHISP_MESSAGE", "OnMessage")
  mod:RegisterMessage("WHISP_SKIN", "OnSkinUpdate")
end

function mod:OnDisable()
end

function mod:OnSkinUpdate()
  Whisp:SkinMyFrame(brokerFrame)
end

function mod:OnMessage()
  if Whisp.lastSender and Whisp.db.realm.chatHistory[Whisp.lastSender] then
    local v = Whisp.db.realm.chatHistory[Whisp.lastSender]
    local inc = v.incoming[#v.incoming]
    local color = inc and Whisp.db.char.colorIncoming or Whisp.db.char.colorOutgoing
    mod.brokerFrame.obj.text = Whisp:Colorise(Whisp.lastSender, Whisp:HexColor(color))
  else
    mod.brokerFrame.obj.text = Whisp.lastSender and Whisp.lastSender or "Whisp"
  end
end

function mod:CreateLDBObject()
  mod.brokerFrame.obj = ldb:NewDataObject("Whisp", {
    type = "data source",
    text = "Whisp",
    icon = "Interface\\AddOns\\Whisp\\icon",
    label = "Whisp",
    OnClick = function(frame, button)
        if button == "RightButton" then
          Whisp:OpenConfig()
        else
          if Whisp.lastSender then
            if IsControlKeyDown() then
              Whisp:ShowLogFrame(self.lastSender)
            else
              ChatFrameEditBox:Show()
              ChatFrameEditBox:SetText("/w "..Whisp.lastSender.." ")
              ChatEdit_ParseText(ChatFrameEditBox, 0)
            end
          end
        end
      end,
    OnEnter = function(frame)
        if not tablet:IsRegistered("Broker_Whisp") then self:RegTablet() end      
        tablet:Refresh("Broker_Whisp")
        tablet:Open(frame, "Broker_Whisp")
      end,
    OnLeave = function()
        --tablet:Close("Broker_Whisp")
      end,
  })
end

function mod:RegTablet()
  tablet:Register("Broker_Whisp", 
      'point', mod.Point,
      'relativePoint', mod.RelPoint,
      'children', function()
            tablet:SetHint(L["Click to reply\nControl-click to open log"])
              tablet:SetTitle("Whisp")
              mod:UpdateBrokerTooltip()
          end,
      'clickable', true,
      'strata', "HIGH"
      )
end

function mod:GetScaledCursorPosition()
  local x, y = GetCursorPosition()
  local scale = UIParent:GetEffectiveScale()
  return x / scale, y / scale, scale
end

function mod:RelPoint()
 local x,y = mod:GetScaledCursorPosition()
 local xp = mod:XPoint(x)
 return (y<GetScreenHeight() / 2) and "TOP"..xp or "BOTTOM"..xp
end

function mod:Point()
 local x,y = mod:GetScaledCursorPosition()
 local xp = mod:XPoint(x)
 return (y<GetScreenHeight() / 2) and "BOTTOM"..xp or "TOP"..xp
end

function mod:XPoint(x)
  if x < GetScreenWidth()/2 - GetScreenWidth()/4 then
    return "LEFT"
  elseif x > GetScreenWidth()/2 + GetScreenWidth()/4 then
    return "RIGHT"
  end
  return ""
end


function mod:CreateMyFrame(name, width)
  local frame = CreateFrame("Frame", "WHISP_"..name, UIParent)
  frame:ClearAllPoints()
  -- Size is required to be set or SetPoint() won't work later on
  frame:SetWidth(width)  
  frame:SetHeight(20)
  frame:SetMinResize(200,20)
  frame:SetFrameLevel(GameTooltip:GetFrameLevel() + 10)
  -- Setup textfields
  frame.text = frame:CreateFontString("WHISP_EDITBOXTEXT","OVERLAY","ChatFontNormal")
  frame.text:SetJustifyH("LEFT")
  frame.text:SetJustifyV("TOP")
  frame.text:SetTextColor(1.0,1.0,1.0,1.0)
  -- Set background and border
  frame:SetFrameStrata("TOOLTIP")
  frame:Hide()
  return frame
end

function mod:GetScaledCursorPosition()
  local x, y = GetCursorPosition()
  local scale = UIParent:GetEffectiveScale()
  return x / scale, y / scale, scale
end

function mod:UpdateBrokerTooltip()
    if not self.skinned then 
      self.skinned = true
      Whisp:SkinMyFrame(brokerFrame) 
    end
            
    -- Setup clickable header
    local cat = tablet:AddCategory(
        'text', L["Timeframe"],
        'text2', L[mod.db.char.tooltipTimeFrame],
        'textR', 0.6, 'textG', 0.6, 'textB', 1,
        'text2R', 1, 'text2G', 0.8, 'text2B', 0,
        'columns', 2,
        'func', function() -- Allow click to cycle through timeframe
              if mod.db.char.tooltipTimeFrame == "Session" then
                mod.db.char.tooltipTimeFrame = "Hour"
            elseif mod.db.char.tooltipTimeFrame == "Hour" then
                  mod.db.char.tooltipTimeFrame = "Day"
              elseif mod.db.char.tooltipTimeFrame == "Day" then
                  mod.db.char.tooltipTimeFrame = "Week"
              elseif mod.db.char.tooltipTimeFrame == "Week" then
                  mod.db.char.tooltipTimeFrame = "All"
              else
                  mod.db.char.tooltipTimeFrame = "Session"
              end
            end,
        'hasCheck', true,
        'checked', true,
        'checkIcon', "Interface\\Buttons\\UI-PlusButton-Up",
        'hideBlankLine', true
    )
    cat:AddLine()
    local cat = tablet:AddCategory(
        'text', L["Sort by"],
        'text2', L[mod.db.char.tooltipSort],
        'textR', 0.6, 'textG', 0.6, 'textB', 1,
        'text2R', 1, 'text2G', 0.8, 'text2B', 0,
        'columns', 2,
        'func', function() -- Allow click to cycle through timeframe
                if mod.db.char.tooltipSort == "Time" then
                    mod.db.char.tooltipSort = "Name"
                else
                    mod.db.char.tooltipSort = "Time"
                end
            end,
        'hasCheck', true,
        'checked', true,
        'checkIcon', "Interface\\Buttons\\UI-PlusButton-Up",
        'hideBlankLine', true
    )
    cat:AddLine()

  -- Setup data
    local contacts = {}
    for i,v in pairs(Whisp.db.realm.chatHistory) do
        tinsert(contacts, {time = v.time[#v.time], plr = i, inc = v.incoming[#v.incoming]})
    end
    if mod.db.char.tooltipSort == "Name" then
        table.sort(contacts, function(a,b) return a.plr<b.plr end)
    else
        table.sort(contacts, function(a,b) return a.time>b.time end)
    end
  
    local tf = mod.db.char.tooltipTimeFrame
    for i=1, #contacts do
        local inc = contacts[i].inc
        local plr = contacts[i].plr
        local systime = contacts[i].time
        local skip = false
    
        if   ((tf == "Session") and (systime < Whisp.sessionStart))
            or ((tf == "Hour")    and (time() - systime > 3600))
            or ((tf == "Day")     and (time() - systime > 86400))
            or ((tf == "Week")    and (time() - systime > 604800))
        then
            skip = true
            
        end
    
        if not skip then
            if (tf == "Week") or (tf == "All") then
              systime = date("%d/%m - %H:%M:%S", systime)
            else
              systime = date("%H:%M:%S", systime)
            end
            local color = inc and Whisp.db.char.colorIncoming or Whisp.db.char.colorOutgoing
            cat:AddLine(
                'text', systime,
                'text2', plr,
                'text2R', color[1],
                'text2G', color[2],
                'text2B', color[3],
                'func', function(plr) 
                      if IsControlKeyDown() then
                          Whisp:ShowLogFrame(plr)
                      else
                          ChatFrameEditBox:Show()
                          ChatFrameEditBox:SetText("/w "..plr.." ")
                          ChatEdit_ParseText(ChatFrameEditBox, 0)
                      end
                  end,
                'arg1', plr,
                'onEnterFunc', function(plr) mod:ShowBrokerFrame(plr) end,
                'onEnterArg1', plr,
                'onLeaveFunc', function() mod:HideBrokerFrame() end,
                'onLeaveArg1', plr
            )
        end
    end
    tablet.tooltip.registration.maxHeight = UIParent:GetTop()/3
end

function mod:ShowBrokerFrame(plr)
  if not plr then return end
  local tip = tablet.tooltip
  brokerFrame:ClearAllPoints()
  brokerFrame:SetPoint(mod:Point(), tip, mod:RelPoint())
  Whisp:UpdateMyFrame(brokerFrame, plr)
  brokerFrame:Show()
end

function mod:HideBrokerFrame()
  brokerFrame:Hide()
end

function mod:GetOptions()
  return options
end

function mod:Info()
  return L["Allows you to quickly review your message history using your favourite Broker display.\n\nFor use with FuBar: Download the Broker2Fubar addon."]
end